from copy import deepcopy
import networkx as nx

from src.Model.Data_Storage.Filters_Model import Filters_Infos
from src.Model.Data_Storage.DataSingleton_Model import DataSingleton

class ConnGraph_Infos(metaclass=DataSingleton):
    """
    A class to manage and store information about connectivity graphs

        - Saving & Loading Data
        - Compute Filtering Data
        - Compute graph-theoretic measures
    """
    
    def __init__(self):

        self.filters = Filters_Infos()
        self.InitReset()

    # ===================================
    # ===== Init / Reset Graph Data =====
    # ===================================

    def InitReset(self):
        """
        Reset or initialize the graph data to their default states.
        """

        self.nxGraph = None
        self.nxGraphBinary = None
        self.dictGraph = None

        self.allGraphValues = []

        self.idName = {}  # Link between ID and Name of each area -> {ID_Node: Name}
        self.areaInfos = {}  # Area Informations
                             # {Name: {ID, RGBA, Side, MajorRegion, ID_Opposite, 3D_Coos}} -> FLUT File
        self.areasOrder = []  # Order of areas (with blanks)
        self.colorMajorRegions = {}
        self.numberOfNodes = None
        
        self.valuesRank = {}
        self.numberOfNodesInData = None
        self.edgesValues = {}  # Value of each edge in the graph -> {ID_Node: {ID_Next: Value}}
        self.edgesValuesFiltered = {}
        self.edgesValues_withoutDuplicata = {}
        self.edgesTypeConnexion = {}  # Connexion Type -> ID_Node: {ID_Next: Value}
                                      # Value in ("Contralateral", "Homotopic", "Ipsilateral", "Other")
        self.edgesTypeConnexionFiltered = {}
        self.numberOfEdges = 0

        self.minMax = None
        self.plotMinMax = None
        self.absMinMax = None

        self.adjacencyMatrix = []

    def SaveCurrentConnGraph(self):
        """
        Save the current state of the connectivity graph.

        Returns
        -------
        dict
            A dictionary containing deep copies of the current graph data.
        """

        return {
            "nxGraph": deepcopy(self.nxGraph),
            "dictGraph": deepcopy(self.dictGraph),

            "allGraphValues": deepcopy(self.allGraphValues),
            "idName": deepcopy(self.idName),
            "areaInfos": deepcopy(self.areaInfos),
            "areasOrder": deepcopy(self.areasOrder),
            "colorMajorRegions": deepcopy(self.colorMajorRegions),

            "valuesRank": deepcopy(self.valuesRank),
            "edgesValues": deepcopy(self.edgesValues),
            "edgesValues_withoutDuplicata": deepcopy(self.edgesValues_withoutDuplicata),
            "edgesTypeConnexion": deepcopy(self.edgesTypeConnexion),

            "minMax": deepcopy(self.minMax),
            "plotMinMax": deepcopy(self.plotMinMax),
            "absMinMax": deepcopy(self.absMinMax),

            "numberOfNodes": self.numberOfNodes,
            "numberOfEdges": self.numberOfEdges,
            "adjacencyMatrix": deepcopy(self.adjacencyMatrix)
        }

    def LoadSaveConnGraph(self, connGraphSave: dict):
        """
        Load a previously saved connectivity graph state.

        Parameters
        ----------
        connGraphSave : dict
            A dictionary containing saved graph data.
        """

        self.nxGraph = connGraphSave["nxGraph"]
        self.dictGraph = connGraphSave["dictGraph"]

        self.allGraphValues = connGraphSave["allGraphValues"]
        self.idName = connGraphSave["idName"]
        self.areaInfos = connGraphSave["areaInfos"]
        self.areasOrder = connGraphSave["areasOrder"]
        self.colorMajorRegions = connGraphSave["colorMajorRegions"]

        self.valuesRank = connGraphSave["valuesRank"]
        self.edgesValues = connGraphSave["edgesValues"]
        self.edgesValues_withoutDuplicata = connGraphSave["edgesValues_withoutDuplicata"]
        self.edgesTypeConnexion = connGraphSave["edgesTypeConnexion"]

        self.minMax = connGraphSave["minMax"]
        self.plotMinMax = connGraphSave["plotMinMax"]
        self.absMinMax = connGraphSave["absMinMax"]

        self.numberOfNodes = connGraphSave["numberOfNodes"]
        self.numberOfEdges = connGraphSave["numberOfEdges"]
        self.adjacencyMatrix = connGraphSave["adjacencyMatrix"]

        # If the filters isn't totally initialize -> not SetEdgesValuesFiltered
        if self.filters.discardInterRegConn:
            self.SetEdgesValuesFiltered()
    

    # ========================
    # ===== Set Variable =====
    # ========================

    def SetGraph(self, nxGraph: nx.Graph):
        """
        Set the networkx graph object and update the internal dictionary representation.

        Parameters
        ----------
        nxGraph : nx.Graph
            The networkx graph object to be set.
        """

        self.nxGraph = nxGraph
        self.dictGraph = nx.to_dict_of_dicts(self.nxGraph)

        if nx.empty_graph(self.nxGraph):
            self.adjacencyMatrix = nx.adjacency_matrix(self.nxGraph).todense()

    def SetTypeConnexion(self):
        """
        Set the type of connections (Ipsilateral, Contralateral, Homotopic, Other) 
        for all edges in the graph based on the area information.
        """

        for edge_source, edgesDestValue in self.edgesValues.items():

            name_source = self.idName[edge_source]
            if edge_source not in self.edgesTypeConnexion:
                self.edgesTypeConnexion[edge_source] = {}

            for edge_dest in edgesDestValue.keys():

                name_dest = self.idName[edge_dest]

                # Ipsilateral Connexion
                if self.areaInfos[name_source]["Side"] == self.areaInfos[name_dest]["Side"]:
                    self.edgesTypeConnexion[edge_source][edge_dest] = "Ipsilateral"

                elif self.areaInfos[name_source]["Side"] != self.areaInfos[name_dest]["Side"]:

                    # 2 -> 0 or 0 -> 2 => Other Connexion
                    if self.areaInfos[name_source]["Side"] == 0 or self.areaInfos[name_dest]["Side"] == 0:
                        self.edgesTypeConnexion[edge_source][edge_dest] = "Other"

                    # Contralateral or Homotopic Connexion
                    else:
                        if self.areaInfos[name_source]["ID_Opposite"] == self.areaInfos[name_dest]["ID"]:
                            self.edgesTypeConnexion[edge_source][edge_dest] = "Homotopic"
                        else:
                            self.edgesTypeConnexion[edge_source][edge_dest] = "Contralateral"


    # ========================
    # ===== Get Variable =====
    # ========================

    def GetAllValues(self):
        """
        Return a list of all edge weights in the graph.

        Returns
        -------
        list
            A list of edge weights.
        """

        return list(self.edgesValues_withoutDuplicata.values())

    def GetAllAbsValues(self):
        """
        Return a list of all absolute edge weights in the graph.

        Returns
        -------
        list
            A list of absolute edge weights.
        """

        return [abs(value) for value in list(self.edgesValues_withoutDuplicata.values())]

    def GetRegionNameWithID(self, ID: int):
        """
        Return the name corresponding to a given region ID.

        Parameters
        ----------
        ID : int
            The ID of the region.

        Returns
        -------
        str
            The name of the region.
        """

        return self.idName[ID]

    def GetRegionIDWithName(self, name: str):
        """
        Return the ID corresponding to a given region name.

        Parameters
        ----------
        name : str
            The name of the region.

        Returns
        -------
        int or None
            The ID of the region, or None if not found.
        """

        for regionID, regionName in self.idName.items():
            if regionName == name:
                return regionID
        return None

    def GetAreasInWiderArea(self, widerAreaName : str):
        """
        Return a list of areas within a specified wider area (MajorRegion).

        Parameters
        ----------
        widerAreaName : str
            The name of the wider area (MajorRegion).

        Returns
        -------
        list
            A list of area names within the specified wider area.
        """

        areas = []
        for areaName, areaDetails in self.areaInfos.items():
            if widerAreaName == areaDetails["MajorRegion"]:
                areas.append(areaName)

        return areas

    def GetAllInterRegionalName(self):
        """
        Return a list of all inter-regional names (MajorRegions) present in the graph.

        Returns
        -------
        list
            A list of unique inter-regional names.
        """

        interRegionalNames = []
        for _, info in self.areaInfos.items():
            name = info["MajorRegion"]
            if name not in interRegionalNames :
                interRegionalNames.append(name)


        return interRegionalNames

    # ==============================
    # ===== Filtered Variables =====
    # ==============================
    
    def SetEdgesValuesFiltered(self):
        """
        Apply filters to the edges and update edgesValuesFiltered and edgesTypeConnexionFiltered.
        """

        self.edgesValuesFiltered = {}
        self.edgesTypeConnexionFiltered = {}
        
        if self.nxGraph:
            self.nxGraph.remove_edges_from(list(self.nxGraph.edges))

        # Sort element respecting filters
        for idEdge_source, destValue in self.edgesValues.items():

            nameEdge_source = self.GetRegionNameWithID(idEdge_source)
            for idEdge_dest, valueEdge in destValue.items():

                # ----- Sorted with the Weight / Rank -----

                # Relative Weight
                if self.filters.discardWeight:
                    if valueEdge < self.filters.weightBetween_threshold[0] or \
                        valueEdge > self.filters.weightBetween_threshold[1]:
                        continue
                
                # Absolute Weight
                elif self.filters.discardAbsWeight:
                    if abs(valueEdge) < self.filters.absWeightBetween_threshold[0] or \
                        abs(valueEdge) > self.filters.absWeightBetween_threshold[1]:
                        continue

                # Rank
                elif self.filters.discardRank:
                    if self.valuesRank[valueEdge] < self.filters.rankBetween_threshold[0] or \
                        self.valuesRank[valueEdge] > self.filters.rankBetween_threshold[1]:
                        continue

                nameEdge_dest = self.GetRegionNameWithID(idEdge_dest)

                # Sorted with Major Regions
                majorRegion_source = self.areaInfos[nameEdge_source]["MajorRegion"]
                majorRegion_dest = self.areaInfos[nameEdge_dest]["MajorRegion"]

                if not self.filters.discardInterRegConn[(majorRegion_source, majorRegion_dest)]:
                    continue

                # Sorted with Connection Type
                connType_sourceDest = self.edgesTypeConnexion[idEdge_source][idEdge_dest]

                if not self.filters.contralateral_connType:
                    if connType_sourceDest == "Contralateral":
                        continue

                if not self.filters.homotopic_connType:
                    if connType_sourceDest == "Homotopic":
                        continue
                
                if not self.filters.ipsilateral_connType:
                    if connType_sourceDest == "Ipsilateral":
                        continue

                if not self.filters.other_connType:
                    if connType_sourceDest == "Other":
                        continue

                # Initializes the dictionnary
                if idEdge_source not in self.edgesValuesFiltered:
                    self.edgesValuesFiltered[idEdge_source] = {} 
                    self.edgesTypeConnexionFiltered[idEdge_source] = {} 

                # Update Variables
                self.edgesValuesFiltered[idEdge_source][idEdge_dest] = valueEdge
                self.edgesTypeConnexionFiltered[idEdge_source][idEdge_dest] = connType_sourceDest
                self.nxGraph.add_weighted_edges_from([(nameEdge_source, nameEdge_dest, abs(valueEdge))])

    # =========================================
    # ===== Method with by MainController =====
    # =========================================

    # ----- List Part -----

    def GetEdgesDetails_List(self):
        """
        Get a list of detailed information for each edge in the filtered graph.

        Returns
        -------
        list
            A list of lists, where each inner list contains details for an edge, including:
            - Source node ID
            - Destination node ID
            - Source region name
            - Destination region name
            - Source major region
            - Destination major region
            - Type of connection
            - Edge weight
            - Edge rank
        """

        edgesDetails = []

        for edge_source, nextEdgeValues in self.edgesValuesFiltered.items():
            for edge_dest, value in nextEdgeValues.items():

                area_1 = self.GetRegionNameWithID(edge_source)
                area_2 = self.GetRegionNameWithID(edge_dest)

                region_1 = self.areaInfos[area_1]["MajorRegion"]
                region_2 = self.areaInfos[area_2]["MajorRegion"]

                connectionType = self.edgesTypeConnexion[edge_source][edge_dest]

                rank = self.valuesRank[value]

                details = [edge_source, edge_dest, area_1, area_2, region_1, region_2, connectionType, value, rank]

                edgesDetails.append(details)

        return edgesDetails


    # ----- Pie Chart Part -----    

    def GetAllNameWithConnectivity_PieChart(self):
        """
        Get a list of region names that have connectivity in the filtered graph.

        Returns
        -------
        list
            A list of region names.
        """

        namesWithConnectivity = []
        for id in self.edgesValuesFiltered.keys():
            namesWithConnectivity.append(self.GetRegionNameWithID(id))

        return namesWithConnectivity

    def GetAllConnectivityWithName_PieChart(self, name: str):
        """
        Get a dictionary of connected regions with their connectivity values and colors for a specific region.

        Parameters
        ----------
        name : str
            The name of the region to check connectivity for.

        Returns
        -------
        dict
            A dictionary where the keys are connected region names, and the values are tuples containing
            the absolute connectivity value and the region's RGBA color.
        """

        id_name = self.GetRegionIDWithName(name)

        connectivities = {}
        for edge_dest, value in self.edgesValuesFiltered[id_name].items():
            nextName = self.GetRegionNameWithID(edge_dest)
            connectivities[nextName] = (abs(value), self.areaInfos[nextName]["RGBA"])

        return connectivities

    def GetAllMajorRegionsWithName_PieChart(self, name: str):
        """
        Get a dictionary of major regions with their total connectivity values for a specific region.

        Parameters
        ----------
        name : str
            The name of the region to check major region connectivity for.

        Returns
        -------
        dict
            A dictionary where the keys are major region names, and the values are the total connectivity values.
        """

        id_name = self.GetRegionIDWithName(name)

        majorRegions = {}
        for edge_dest, edge_value in self.edgesValuesFiltered[id_name].items():

            majorRegion = self.areaInfos[self.GetRegionNameWithID(edge_dest)]["MajorRegion"]
            if majorRegion not in majorRegions:
                majorRegions[majorRegion] = 0.0

            majorRegions[majorRegion] += abs(edge_value)

        return majorRegions

    def GetAllConnectionTypeWithName_PieChart(self, name: str):
        """
        Get a dictionary of connection types with their total values for a specific region.

        Parameters
        ----------
        name : str
            The name of the region to check connection types for.

        Returns
        -------
        dict
            A dictionary where the keys are connection types (Ipsilateral, Contralateral, Homotopic, Other),
            and the values are the total connectivity values for each type.
        """

        id_name = self.GetRegionIDWithName(name)

        connectionsType = {"Ipsilateral": 0.0, "Contralateral": 0.0, "Homotopic": 0.0, "Other": 0.0}
        for edge_dest, connectionType in self.edgesTypeConnexionFiltered[id_name].items():
            connectionsType[connectionType] += abs(self.edgesValues[id_name][edge_dest])

        return connectionsType


    # ----- GT Measures -----

    def local_efficiency(self, graph):
        """
        Calculate the local efficiency for each node in the graph.

        Parameters
        ----------
        graph : nx.Graph
            The graph for which to calculate local efficiency.

        Returns
        -------
        dict
            A dictionary where the keys are node IDs, and the values are their local efficiency values.
        """
        efficiency = {}

        for node in graph.nodes():
            # Determine the neighbors
            neighbors = list(graph.neighbors(node))

            if len(neighbors) < 2:
                efficiency[node] = 0
            else:
                # Create the subgraph of neighbors
                subgraph = graph.subgraph(neighbors)
                
                # Ensure the subgraph is connected before calculating global efficiency
                if nx.is_connected(subgraph):
                    efficiency[node] = nx.global_efficiency(subgraph)
                else:
                    # Calculate efficiency for each connected component
                    component_efficiencies = []
                    for component in nx.connected_components(subgraph):
                        component_subgraph = subgraph.subgraph(component)
                        component_efficiencies.append(nx.global_efficiency(component_subgraph))
                    efficiency[node] = sum(component_efficiencies) / len(component_efficiencies)

        return efficiency

    def eccentricity(self, graph, weight):
        """
        Calculate the eccentricity for each node in the graph, considering edge weights.

        Parameters
        ----------
        graph : nx.Graph
            The graph for which to calculate eccentricity.
        weight : str or None
            The name of the edge attribute used as weight. If None, treats the graph as unweighted.

        Returns
        -------
        dict
            A dictionary where the keys are node IDs, and the values are their eccentricity values.
        """
        eccentricity = {}
        
        # Iterate over each connected component of the graph
        for component in nx.connected_components(graph):
            subgraph = graph.subgraph(component)
            
            # Calculate the lengths of the shortest paths with weights
            path_lengths = dict(nx.all_pairs_dijkstra_path_length(subgraph, weight=weight))
            
            # Calculate eccentricity for subgraph
            subgraph_eccentricity = {}

            for node in subgraph.nodes():
                max_distance = max(path_lengths[node].values(), default=0)
                subgraph_eccentricity[node] = max_distance
            
            eccentricity.update(subgraph_eccentricity)
        
        return eccentricity
    
    def GetLocalMeasures(self, localMeasure):
        """
        Retrieve local graph-theoretic measures for each node in the graph.

        Parameters
        ----------
        localMeasure : str
            The name of the local measure to retrieve. Must match a case in this method.

        Returns
        -------
        dict
            A dictionary where the keys are node IDs, and the values are the computed measure values.
        """
        match localMeasure:
            # ---- Weighted ----

            case "degree (weighted)":
                return dict(self.nxGraph.degree(weight='weight'))
            case "cluster coef (weighted)":
                return nx.clustering(self.nxGraph, weight='weight')
            case "local efficiency (weighted)":
                return self.local_efficiency(self.nxGraph)
            case "betweenness centrality (weighted)":
                return nx.betweenness_centrality(self.nxGraph, weight="weight", normalized=False)
            case "eigenvector centrality (weighted)":
                try:
                    return nx.eigenvector_centrality(self.nxGraph, max_iter=1000, tol=1e-5, weight='weight')
                except nx.PowerIterationFailedConvergence:
                    return nx.eigenvector_centrality_numpy(self.nxGraph, weight='weight')
            case "eccentricity (weighted)":
                return self.eccentricity(self.nxGraph, "weight")

            # ---- Binary ----
            case "degree (binary)":
                return dict(self.nxGraphBinary.degree())
            case "cluster coef (binary)":
                return nx.clustering(self.nxGraphBinary)
            case "local efficiency (binary)":
                return self.local_efficiency(self.nxGraphBinary)
            case "betweenness centrality (binary)":
                return nx.betweenness_centrality(self.nxGraphBinary, normalized=False)
            case "eigenvector centrality (binary)":
                return nx.eigenvector_centrality(self.nxGraphBinary)
            case "eccentricity (binary)":
                return self.eccentricity(self.nxGraphBinary, None)


class DataSingleton(type):
    """
    Singleton Structure (Metaclass Type)
    """

    _instances = {}

    def __call__(cls, *args, **kwargs):

        if cls not in cls._instances:
            cls._instances[cls] = super(DataSingleton, cls).__call__(*args, **kwargs)
            
        return cls._instances[cls]


# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file '.\ressources\QT_ui\ImportWindow.ui'
#
# Created by: PyQt5 UI code generator 5.15.10
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_ImportWindow(object):
    def setupUi(self, ImportWindow):
        ImportWindow.setObjectName("ImportWindow")
        ImportWindow.resize(1800, 900)
        ImportWindow.setMinimumSize(QtCore.QSize(0, 0))
        ImportWindow.setMaximumSize(QtCore.QSize(16777215, 16777215))
        ImportWindow.setAutoFillBackground(True)
        self.centralwidget = QtWidgets.QWidget(ImportWindow)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.centralwidget.sizePolicy().hasHeightForWidth())
        self.centralwidget.setSizePolicy(sizePolicy)
        self.centralwidget.setMaximumSize(QtCore.QSize(16777215, 16777215))
        self.centralwidget.setObjectName("centralwidget")
        self.gridLayout = QtWidgets.QGridLayout(self.centralwidget)
        self.gridLayout.setObjectName("gridLayout")
        self.graphSection_Widget = QtWidgets.QWidget(self.centralwidget)
        self.graphSection_Widget.setMinimumSize(QtCore.QSize(0, 0))
        self.graphSection_Widget.setObjectName("graphSection_Widget")
        self.gridLayout_9 = QtWidgets.QGridLayout(self.graphSection_Widget)
        self.gridLayout_9.setContentsMargins(-1, 7, -1, -1)
        self.gridLayout_9.setObjectName("gridLayout_9")
        self.graphControl_Widget = QtWidgets.QWidget(self.graphSection_Widget)
        self.graphControl_Widget.setObjectName("graphControl_Widget")
        self.gridLayout_2 = QtWidgets.QGridLayout(self.graphControl_Widget)
        self.gridLayout_2.setContentsMargins(0, 0, 0, 0)
        self.gridLayout_2.setObjectName("gridLayout_2")
        self.limitedGraph_TabWidget = QtWidgets.QTabWidget(self.graphControl_Widget)
        self.limitedGraph_TabWidget.setMinimumSize(QtCore.QSize(0, 0))
        self.limitedGraph_TabWidget.setMaximumSize(QtCore.QSize(16777215, 16777215))
        self.limitedGraph_TabWidget.setTabBarAutoHide(False)
        self.limitedGraph_TabWidget.setObjectName("limitedGraph_TabWidget")
        self.threshodTab_Widget = QtWidgets.QWidget()
        self.threshodTab_Widget.setObjectName("threshodTab_Widget")
        self.gridLayout_12 = QtWidgets.QGridLayout(self.threshodTab_Widget)
        self.gridLayout_12.setObjectName("gridLayout_12")
        self.slider_Widget = QtWidgets.QWidget(self.threshodTab_Widget)
        self.slider_Widget.setObjectName("slider_Widget")
        self.gridLayout_11 = QtWidgets.QGridLayout(self.slider_Widget)
        self.gridLayout_11.setObjectName("gridLayout_11")
        spacerItem = QtWidgets.QSpacerItem(20, 834, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.gridLayout_11.addItem(spacerItem, 1, 1, 1, 1)
        self.minSlider_Label = QtWidgets.QLabel(self.slider_Widget)
        self.minSlider_Label.setMinimumSize(QtCore.QSize(0, 0))
        self.minSlider_Label.setObjectName("minSlider_Label")
        self.gridLayout_11.addWidget(self.minSlider_Label, 2, 1, 1, 1)
        self.maxSlider_Label = QtWidgets.QLabel(self.slider_Widget)
        self.maxSlider_Label.setMinimumSize(QtCore.QSize(0, 0))
        self.maxSlider_Label.setObjectName("maxSlider_Label")
        self.gridLayout_11.addWidget(self.maxSlider_Label, 0, 1, 1, 1)
        self.threshold_Slider = QtWidgets.QSlider(self.slider_Widget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.threshold_Slider.sizePolicy().hasHeightForWidth())
        self.threshold_Slider.setSizePolicy(sizePolicy)
        self.threshold_Slider.setOrientation(QtCore.Qt.Vertical)
        self.threshold_Slider.setTickPosition(QtWidgets.QSlider.NoTicks)
        self.threshold_Slider.setObjectName("threshold_Slider")
        self.gridLayout_11.addWidget(self.threshold_Slider, 0, 0, 3, 1)
        self.gridLayout_11.setColumnStretch(0, 30)
        self.gridLayout_11.setColumnStretch(1, 70)
        self.gridLayout_12.addWidget(self.slider_Widget, 0, 0, 1, 1)
        self.thresholdDisplay_Widget = QtWidgets.QWidget(self.threshodTab_Widget)
        self.thresholdDisplay_Widget.setMinimumSize(QtCore.QSize(0, 0))
        self.thresholdDisplay_Widget.setObjectName("thresholdDisplay_Widget")
        self.horizontalLayout_4 = QtWidgets.QHBoxLayout(self.thresholdDisplay_Widget)
        self.horizontalLayout_4.setContentsMargins(0, 0, 0, 0)
        self.horizontalLayout_4.setObjectName("horizontalLayout_4")
        self.threshold_Label = QtWidgets.QLabel(self.thresholdDisplay_Widget)
        self.threshold_Label.setMinimumSize(QtCore.QSize(0, 0))
        self.threshold_Label.setObjectName("threshold_Label")
        self.horizontalLayout_4.addWidget(self.threshold_Label)
        self.thresholdValue_DoubleSpinBox = QtWidgets.QDoubleSpinBox(self.thresholdDisplay_Widget)
        self.thresholdValue_DoubleSpinBox.setObjectName("thresholdValue_DoubleSpinBox")
        self.horizontalLayout_4.addWidget(self.thresholdValue_DoubleSpinBox)
        self.horizontalLayout_4.setStretch(0, 2)
        self.horizontalLayout_4.setStretch(1, 4)
        self.gridLayout_12.addWidget(self.thresholdDisplay_Widget, 1, 0, 1, 1)
        self.rankDisplay_Widget = QtWidgets.QFrame(self.threshodTab_Widget)
        self.rankDisplay_Widget.setFrameShape(QtWidgets.QFrame.StyledPanel)
        self.rankDisplay_Widget.setFrameShadow(QtWidgets.QFrame.Raised)
        self.rankDisplay_Widget.setObjectName("rankDisplay_Widget")
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout(self.rankDisplay_Widget)
        self.horizontalLayout_2.setContentsMargins(0, 0, 0, 0)
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.rank_Label = QtWidgets.QLabel(self.rankDisplay_Widget)
        self.rank_Label.setObjectName("rank_Label")
        self.horizontalLayout_2.addWidget(self.rank_Label)
        self.rankValue_Label = QtWidgets.QLabel(self.rankDisplay_Widget)
        self.rankValue_Label.setText("")
        self.rankValue_Label.setObjectName("rankValue_Label")
        self.horizontalLayout_2.addWidget(self.rankValue_Label)
        self.horizontalLayout_2.setStretch(0, 2)
        self.horizontalLayout_2.setStretch(1, 4)
        self.gridLayout_12.addWidget(self.rankDisplay_Widget, 2, 0, 1, 1)
        self.limitedGraph_TabWidget.addTab(self.threshodTab_Widget, "")
        self.tablesTab_Widget = QtWidgets.QWidget()
        self.tablesTab_Widget.setObjectName("tablesTab_Widget")
        self.gridLayout_10 = QtWidgets.QGridLayout(self.tablesTab_Widget)
        self.gridLayout_10.setObjectName("gridLayout_10")
        self.percentageTable_TableWidget = QtWidgets.QTableWidget(self.tablesTab_Widget)
        self.percentageTable_TableWidget.setLineWidth(0)
        self.percentageTable_TableWidget.setObjectName("percentageTable_TableWidget")
        self.percentageTable_TableWidget.setColumnCount(2)
        self.percentageTable_TableWidget.setRowCount(0)
        item = QtWidgets.QTableWidgetItem()
        self.percentageTable_TableWidget.setHorizontalHeaderItem(0, item)
        item = QtWidgets.QTableWidgetItem()
        self.percentageTable_TableWidget.setHorizontalHeaderItem(1, item)
        self.gridLayout_10.addWidget(self.percentageTable_TableWidget, 0, 0, 1, 1)
        self.nbTable_TableWidget = QtWidgets.QTableWidget(self.tablesTab_Widget)
        self.nbTable_TableWidget.setLineWidth(0)
        self.nbTable_TableWidget.setObjectName("nbTable_TableWidget")
        self.nbTable_TableWidget.setColumnCount(2)
        self.nbTable_TableWidget.setRowCount(0)
        item = QtWidgets.QTableWidgetItem()
        self.nbTable_TableWidget.setHorizontalHeaderItem(0, item)
        item = QtWidgets.QTableWidgetItem()
        self.nbTable_TableWidget.setHorizontalHeaderItem(1, item)
        self.gridLayout_10.addWidget(self.nbTable_TableWidget, 1, 0, 1, 1)
        self.limitedGraph_TabWidget.addTab(self.tablesTab_Widget, "")
        self.gridLayout_2.addWidget(self.limitedGraph_TabWidget, 1, 0, 1, 1)
        self.gridLayout_9.addWidget(self.graphControl_Widget, 0, 1, 1, 1)
        self.graphDisplay_Widget = QtWidgets.QWidget(self.graphSection_Widget)
        self.graphDisplay_Widget.setObjectName("graphDisplay_Widget")
        self.gridLayout_7 = QtWidgets.QGridLayout(self.graphDisplay_Widget)
        self.gridLayout_7.setObjectName("gridLayout_7")
        self.graphDisplay_Layout = QtWidgets.QGridLayout()
        self.graphDisplay_Layout.setSizeConstraint(QtWidgets.QLayout.SetDefaultConstraint)
        self.graphDisplay_Layout.setObjectName("graphDisplay_Layout")
        self.gridLayout_7.addLayout(self.graphDisplay_Layout, 0, 0, 1, 1)
        self.gridLayout_9.addWidget(self.graphDisplay_Widget, 0, 0, 1, 1)
        self.gridLayout_9.setColumnStretch(0, 6)
        self.gridLayout_9.setColumnStretch(1, 2)
        self.gridLayout_9.setRowStretch(0, 1)
        self.gridLayout.addWidget(self.graphSection_Widget, 1, 0, 1, 1)
        self.importFilesSection_Widget = QtWidgets.QWidget(self.centralwidget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.importFilesSection_Widget.sizePolicy().hasHeightForWidth())
        self.importFilesSection_Widget.setSizePolicy(sizePolicy)
        self.importFilesSection_Widget.setMaximumSize(QtCore.QSize(16777215, 16777215))
        self.importFilesSection_Widget.setLayoutDirection(QtCore.Qt.LeftToRight)
        self.importFilesSection_Widget.setObjectName("importFilesSection_Widget")
        self.gridLayout_6 = QtWidgets.QGridLayout(self.importFilesSection_Widget)
        self.gridLayout_6.setObjectName("gridLayout_6")
        self.importFiles_TabWidget = QtWidgets.QTabWidget(self.importFilesSection_Widget)
        self.importFiles_TabWidget.setMinimumSize(QtCore.QSize(0, 0))
        self.importFiles_TabWidget.setObjectName("importFiles_TabWidget")
        self.filesTab_Widget = QtWidgets.QWidget()
        self.filesTab_Widget.setObjectName("filesTab_Widget")
        self.gridLayout_4 = QtWidgets.QGridLayout(self.filesTab_Widget)
        self.gridLayout_4.setObjectName("gridLayout_4")
        self.previousFiltersFiles_Label = QtWidgets.QLabel(self.filesTab_Widget)
        self.previousFiltersFiles_Label.setObjectName("previousFiltersFiles_Label")
        self.gridLayout_4.addWidget(self.previousFiltersFiles_Label, 14, 0, 1, 2)
        self.errorDataNameFile_Label = QtWidgets.QLabel(self.filesTab_Widget)
        self.errorDataNameFile_Label.setText("")
        self.errorDataNameFile_Label.setObjectName("errorDataNameFile_Label")
        self.gridLayout_4.addWidget(self.errorDataNameFile_Label, 2, 0, 1, 2)
        self.previousFlutFiles_List = QtWidgets.QListWidget(self.filesTab_Widget)
        self.previousFlutFiles_List.setMinimumSize(QtCore.QSize(0, 0))
        self.previousFlutFiles_List.setObjectName("previousFlutFiles_List")
        self.gridLayout_4.addWidget(self.previousFlutFiles_List, 10, 0, 1, 2)
        self.openedNameFile_Label = QtWidgets.QLabel(self.filesTab_Widget)
        self.openedNameFile_Label.setMinimumSize(QtCore.QSize(0, 0))
        self.openedNameFile_Label.setAlignment(QtCore.Qt.AlignCenter)
        self.openedNameFile_Label.setObjectName("openedNameFile_Label")
        self.gridLayout_4.addWidget(self.openedNameFile_Label, 1, 0, 1, 1)
        self.openedFlutFile_Label = QtWidgets.QLabel(self.filesTab_Widget)
        self.openedFlutFile_Label.setObjectName("openedFlutFile_Label")
        self.gridLayout_4.addWidget(self.openedFlutFile_Label, 7, 0, 1, 1)
        self.errorFlutFile_Label = QtWidgets.QLabel(self.filesTab_Widget)
        self.errorFlutFile_Label.setText("")
        self.errorFlutFile_Label.setObjectName("errorFlutFile_Label")
        self.gridLayout_4.addWidget(self.errorFlutFile_Label, 8, 0, 1, 2)
        self.importFiltersFile_Button = QtWidgets.QToolButton(self.filesTab_Widget)
        self.importFiltersFile_Button.setText("")
        self.importFiltersFile_Button.setObjectName("importFiltersFile_Button")
        self.gridLayout_4.addWidget(self.importFiltersFile_Button, 12, 1, 1, 1)
        self.importFlutFile_Button = QtWidgets.QToolButton(self.filesTab_Widget)
        self.importFlutFile_Button.setText("")
        self.importFlutFile_Button.setObjectName("importFlutFile_Button")
        self.gridLayout_4.addWidget(self.importFlutFile_Button, 7, 1, 1, 1)
        self.openedFiltersFile_Label = QtWidgets.QLabel(self.filesTab_Widget)
        self.openedFiltersFile_Label.setMinimumSize(QtCore.QSize(0, 0))
        self.openedFiltersFile_Label.setAlignment(QtCore.Qt.AlignCenter)
        self.openedFiltersFile_Label.setObjectName("openedFiltersFile_Label")
        self.gridLayout_4.addWidget(self.openedFiltersFile_Label, 12, 0, 1, 1)
        self.openedDataFile_Label = QtWidgets.QLabel(self.filesTab_Widget)
        self.openedDataFile_Label.setMinimumSize(QtCore.QSize(0, 0))
        self.openedDataFile_Label.setAlignment(QtCore.Qt.AlignCenter)
        self.openedDataFile_Label.setObjectName("openedDataFile_Label")
        self.gridLayout_4.addWidget(self.openedDataFile_Label, 0, 0, 1, 1)
        self.previousDataFiles_Label = QtWidgets.QLabel(self.filesTab_Widget)
        self.previousDataFiles_Label.setMinimumSize(QtCore.QSize(0, 0))
        self.previousDataFiles_Label.setObjectName("previousDataFiles_Label")
        self.gridLayout_4.addWidget(self.previousDataFiles_Label, 4, 0, 1, 2)
        self.importNameFile_Button = QtWidgets.QToolButton(self.filesTab_Widget)
        self.importNameFile_Button.setText("")
        self.importNameFile_Button.setObjectName("importNameFile_Button")
        self.gridLayout_4.addWidget(self.importNameFile_Button, 1, 1, 1, 1)
        spacerItem1 = QtWidgets.QSpacerItem(10, 10, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.gridLayout_4.addItem(spacerItem1, 6, 0, 1, 2)
        spacerItem2 = QtWidgets.QSpacerItem(10, 10, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.gridLayout_4.addItem(spacerItem2, 3, 0, 1, 2)
        self.previousFlutFiles_Label = QtWidgets.QLabel(self.filesTab_Widget)
        self.previousFlutFiles_Label.setMinimumSize(QtCore.QSize(0, 0))
        self.previousFlutFiles_Label.setObjectName("previousFlutFiles_Label")
        self.gridLayout_4.addWidget(self.previousFlutFiles_Label, 9, 0, 1, 2)
        self.importDataFile_Button = QtWidgets.QToolButton(self.filesTab_Widget)
        self.importDataFile_Button.setText("")
        self.importDataFile_Button.setObjectName("importDataFile_Button")
        self.gridLayout_4.addWidget(self.importDataFile_Button, 0, 1, 1, 1)
        spacerItem3 = QtWidgets.QSpacerItem(10, 10, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.gridLayout_4.addItem(spacerItem3, 11, 0, 1, 2)
        self.previousDataFiles_List = QtWidgets.QListWidget(self.filesTab_Widget)
        self.previousDataFiles_List.setMinimumSize(QtCore.QSize(0, 0))
        self.previousDataFiles_List.setObjectName("previousDataFiles_List")
        self.gridLayout_4.addWidget(self.previousDataFiles_List, 5, 0, 1, 2)
        self.errorFiltersFile_Label = QtWidgets.QLabel(self.filesTab_Widget)
        self.errorFiltersFile_Label.setText("")
        self.errorFiltersFile_Label.setObjectName("errorFiltersFile_Label")
        self.gridLayout_4.addWidget(self.errorFiltersFile_Label, 13, 0, 1, 2)
        self.previousFiltersFiles_List = QtWidgets.QListWidget(self.filesTab_Widget)
        self.previousFiltersFiles_List.setMinimumSize(QtCore.QSize(0, 0))
        self.previousFiltersFiles_List.setObjectName("previousFiltersFiles_List")
        self.gridLayout_4.addWidget(self.previousFiltersFiles_List, 15, 0, 1, 2)
        self.gridLayout_4.setRowStretch(0, 5)
        self.gridLayout_4.setRowStretch(1, 5)
        self.gridLayout_4.setRowStretch(2, 5)
        self.gridLayout_4.setRowStretch(3, 1)
        self.gridLayout_4.setRowStretch(4, 5)
        self.gridLayout_4.setRowStretch(5, 60)
        self.gridLayout_4.setRowStretch(6, 1)
        self.gridLayout_4.setRowStretch(7, 5)
        self.gridLayout_4.setRowStretch(8, 5)
        self.gridLayout_4.setRowStretch(9, 5)
        self.gridLayout_4.setRowStretch(10, 35)
        self.gridLayout_4.setRowStretch(11, 1)
        self.gridLayout_4.setRowStretch(12, 5)
        self.gridLayout_4.setRowStretch(13, 5)
        self.gridLayout_4.setRowStretch(14, 5)
        self.gridLayout_4.setRowStretch(15, 35)
        self.importFiles_TabWidget.addTab(self.filesTab_Widget, "")
        self.gridLayout_6.addWidget(self.importFiles_TabWidget, 0, 0, 1, 1)
        self.validationSection_Widget = QtWidgets.QWidget(self.importFilesSection_Widget)
        self.validationSection_Widget.setObjectName("validationSection_Widget")
        self.horizontalLayout_5 = QtWidgets.QHBoxLayout(self.validationSection_Widget)
        self.horizontalLayout_5.setObjectName("horizontalLayout_5")
        self.errorValidation_Label = QtWidgets.QLabel(self.validationSection_Widget)
        self.errorValidation_Label.setMinimumSize(QtCore.QSize(0, 0))
        self.errorValidation_Label.setText("")
        self.errorValidation_Label.setObjectName("errorValidation_Label")
        self.horizontalLayout_5.addWidget(self.errorValidation_Label)
        self.validation_Button = QtWidgets.QToolButton(self.validationSection_Widget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.validation_Button.sizePolicy().hasHeightForWidth())
        self.validation_Button.setSizePolicy(sizePolicy)
        self.validation_Button.setMinimumSize(QtCore.QSize(0, 0))
        self.validation_Button.setObjectName("validation_Button")
        self.horizontalLayout_5.addWidget(self.validation_Button)
        self.horizontalLayout_5.setStretch(0, 4)
        self.gridLayout_6.addWidget(self.validationSection_Widget, 1, 0, 1, 1)
        self.gridLayout_6.setRowStretch(0, 95)
        self.gridLayout_6.setRowStretch(1, 5)
        self.gridLayout.addWidget(self.importFilesSection_Widget, 0, 1, 2, 1)
        self.informationsSection_Widget = QtWidgets.QWidget(self.centralwidget)
        self.informationsSection_Widget.setContextMenuPolicy(QtCore.Qt.NoContextMenu)
        self.informationsSection_Widget.setObjectName("informationsSection_Widget")
        self.gridLayout_8 = QtWidgets.QGridLayout(self.informationsSection_Widget)
        self.gridLayout_8.setObjectName("gridLayout_8")
        self.openedFiles_Widget = QtWidgets.QWidget(self.informationsSection_Widget)
        self.openedFiles_Widget.setMinimumSize(QtCore.QSize(0, 0))
        self.openedFiles_Widget.setObjectName("openedFiles_Widget")
        self.horizontalLayout = QtWidgets.QHBoxLayout(self.openedFiles_Widget)
        self.horizontalLayout.setContentsMargins(0, 0, 0, 0)
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.openedCurrentFiles_Label = QtWidgets.QLabel(self.openedFiles_Widget)
        self.openedCurrentFiles_Label.setMinimumSize(QtCore.QSize(0, 0))
        self.openedCurrentFiles_Label.setObjectName("openedCurrentFiles_Label")
        self.horizontalLayout.addWidget(self.openedCurrentFiles_Label)
        self.openedCurrentDataFile_Label = QtWidgets.QLabel(self.openedFiles_Widget)
        self.openedCurrentDataFile_Label.setMinimumSize(QtCore.QSize(0, 0))
        self.openedCurrentDataFile_Label.setObjectName("openedCurrentDataFile_Label")
        self.horizontalLayout.addWidget(self.openedCurrentDataFile_Label)
        self.currentDataFile_Label = QtWidgets.QLabel(self.openedFiles_Widget)
        self.currentDataFile_Label.setMinimumSize(QtCore.QSize(0, 0))
        self.currentDataFile_Label.setText("")
        self.currentDataFile_Label.setObjectName("currentDataFile_Label")
        self.horizontalLayout.addWidget(self.currentDataFile_Label)
        self.openedCurrentFlutFile_Label = QtWidgets.QLabel(self.openedFiles_Widget)
        self.openedCurrentFlutFile_Label.setMinimumSize(QtCore.QSize(0, 0))
        self.openedCurrentFlutFile_Label.setObjectName("openedCurrentFlutFile_Label")
        self.horizontalLayout.addWidget(self.openedCurrentFlutFile_Label)
        self.currentFlutFile_Label = QtWidgets.QLabel(self.openedFiles_Widget)
        self.currentFlutFile_Label.setMinimumSize(QtCore.QSize(0, 0))
        self.currentFlutFile_Label.setText("")
        self.currentFlutFile_Label.setObjectName("currentFlutFile_Label")
        self.horizontalLayout.addWidget(self.currentFlutFile_Label)
        self.horizontalLayout.setStretch(0, 2)
        self.horizontalLayout.setStretch(1, 1)
        self.horizontalLayout.setStretch(2, 4)
        self.horizontalLayout.setStretch(3, 1)
        self.horizontalLayout.setStretch(4, 4)
        self.gridLayout_8.addWidget(self.openedFiles_Widget, 0, 0, 1, 1)
        self.fileInformations_Widget = QtWidgets.QWidget(self.informationsSection_Widget)
        self.fileInformations_Widget.setObjectName("fileInformations_Widget")
        self.gridLayout_3 = QtWidgets.QGridLayout(self.fileInformations_Widget)
        self.gridLayout_3.setSizeConstraint(QtWidgets.QLayout.SetDefaultConstraint)
        self.gridLayout_3.setContentsMargins(0, 0, 0, 0)
        self.gridLayout_3.setObjectName("gridLayout_3")
        self.nbConnections_Label = QtWidgets.QLabel(self.fileInformations_Widget)
        self.nbConnections_Label.setMinimumSize(QtCore.QSize(0, 0))
        self.nbConnections_Label.setObjectName("nbConnections_Label")
        self.gridLayout_3.addWidget(self.nbConnections_Label, 1, 0, 1, 1)
        self.nbNodes_Label = QtWidgets.QLabel(self.fileInformations_Widget)
        self.nbNodes_Label.setMinimumSize(QtCore.QSize(0, 0))
        self.nbNodes_Label.setObjectName("nbNodes_Label")
        self.gridLayout_3.addWidget(self.nbNodes_Label, 0, 0, 1, 1)
        self.nbNodesValue_Label = QtWidgets.QLabel(self.fileInformations_Widget)
        self.nbNodesValue_Label.setMinimumSize(QtCore.QSize(0, 0))
        self.nbNodesValue_Label.setText("")
        self.nbNodesValue_Label.setObjectName("nbNodesValue_Label")
        self.gridLayout_3.addWidget(self.nbNodesValue_Label, 0, 2, 1, 1)
        self.sumAbsValue_Label = QtWidgets.QLabel(self.fileInformations_Widget)
        self.sumAbsValue_Label.setMinimumSize(QtCore.QSize(0, 0))
        self.sumAbsValue_Label.setText("")
        self.sumAbsValue_Label.setObjectName("sumAbsValue_Label")
        self.gridLayout_3.addWidget(self.sumAbsValue_Label, 0, 4, 1, 1)
        self.max_Label = QtWidgets.QLabel(self.fileInformations_Widget)
        self.max_Label.setMinimumSize(QtCore.QSize(0, 0))
        self.max_Label.setObjectName("max_Label")
        self.gridLayout_3.addWidget(self.max_Label, 0, 5, 1, 1)
        self.standardDeviation_Label = QtWidgets.QLabel(self.fileInformations_Widget)
        self.standardDeviation_Label.setMinimumSize(QtCore.QSize(0, 0))
        self.standardDeviation_Label.setObjectName("standardDeviation_Label")
        self.gridLayout_3.addWidget(self.standardDeviation_Label, 0, 7, 1, 1)
        self.sumAbs_Label = QtWidgets.QLabel(self.fileInformations_Widget)
        self.sumAbs_Label.setMinimumSize(QtCore.QSize(0, 0))
        self.sumAbs_Label.setObjectName("sumAbs_Label")
        self.gridLayout_3.addWidget(self.sumAbs_Label, 0, 3, 1, 1)
        self.standardDeviationValue_Label = QtWidgets.QLabel(self.fileInformations_Widget)
        self.standardDeviationValue_Label.setMinimumSize(QtCore.QSize(0, 0))
        self.standardDeviationValue_Label.setText("")
        self.standardDeviationValue_Label.setObjectName("standardDeviationValue_Label")
        self.gridLayout_3.addWidget(self.standardDeviationValue_Label, 0, 8, 1, 1)
        self.maxValue_Label = QtWidgets.QLabel(self.fileInformations_Widget)
        self.maxValue_Label.setMinimumSize(QtCore.QSize(0, 0))
        self.maxValue_Label.setText("")
        self.maxValue_Label.setObjectName("maxValue_Label")
        self.gridLayout_3.addWidget(self.maxValue_Label, 0, 6, 1, 1)
        self.nbConnectionsValue_Label = QtWidgets.QLabel(self.fileInformations_Widget)
        self.nbConnectionsValue_Label.setMinimumSize(QtCore.QSize(0, 0))
        self.nbConnectionsValue_Label.setText("")
        self.nbConnectionsValue_Label.setObjectName("nbConnectionsValue_Label")
        self.gridLayout_3.addWidget(self.nbConnectionsValue_Label, 1, 2, 1, 1)
        self.meanAbs_Label = QtWidgets.QLabel(self.fileInformations_Widget)
        self.meanAbs_Label.setMinimumSize(QtCore.QSize(0, 0))
        self.meanAbs_Label.setObjectName("meanAbs_Label")
        self.gridLayout_3.addWidget(self.meanAbs_Label, 1, 3, 1, 1)
        self.meanAbsValue_Label = QtWidgets.QLabel(self.fileInformations_Widget)
        self.meanAbsValue_Label.setMinimumSize(QtCore.QSize(0, 0))
        self.meanAbsValue_Label.setText("")
        self.meanAbsValue_Label.setObjectName("meanAbsValue_Label")
        self.gridLayout_3.addWidget(self.meanAbsValue_Label, 1, 4, 1, 1)
        self.otherValueValue_Label = QtWidgets.QLabel(self.fileInformations_Widget)
        self.otherValueValue_Label.setText("")
        self.otherValueValue_Label.setObjectName("otherValueValue_Label")
        self.gridLayout_3.addWidget(self.otherValueValue_Label, 1, 8, 1, 1)
        self.otherValue_Label = QtWidgets.QLabel(self.fileInformations_Widget)
        self.otherValue_Label.setText("")
        self.otherValue_Label.setObjectName("otherValue_Label")
        self.gridLayout_3.addWidget(self.otherValue_Label, 1, 7, 1, 1)
        self.minValue_Label = QtWidgets.QLabel(self.fileInformations_Widget)
        self.minValue_Label.setMinimumSize(QtCore.QSize(0, 0))
        self.minValue_Label.setText("")
        self.minValue_Label.setObjectName("minValue_Label")
        self.gridLayout_3.addWidget(self.minValue_Label, 1, 6, 1, 1)
        self.min_Label = QtWidgets.QLabel(self.fileInformations_Widget)
        self.min_Label.setMinimumSize(QtCore.QSize(0, 0))
        self.min_Label.setObjectName("min_Label")
        self.gridLayout_3.addWidget(self.min_Label, 1, 5, 1, 1)
        self.gridLayout_8.addWidget(self.fileInformations_Widget, 1, 0, 1, 1)
        self.gridLayout_8.setRowStretch(0, 1)
        self.gridLayout_8.setRowStretch(1, 2)
        self.gridLayout.addWidget(self.informationsSection_Widget, 0, 0, 1, 1)
        self.gridLayout.setColumnStretch(0, 4)
        self.gridLayout.setColumnStretch(1, 1)
        self.gridLayout.setRowStretch(0, 1)
        self.gridLayout.setRowStretch(1, 4)
        ImportWindow.setCentralWidget(self.centralwidget)

        self.retranslateUi(ImportWindow)
        self.limitedGraph_TabWidget.setCurrentIndex(0)
        self.importFiles_TabWidget.setCurrentIndex(0)
        QtCore.QMetaObject.connectSlotsByName(ImportWindow)

    def retranslateUi(self, ImportWindow):
        _translate = QtCore.QCoreApplication.translate
        ImportWindow.setWindowTitle(_translate("ImportWindow", "ImportWindow"))
        self.limitedGraph_TabWidget.setProperty("class", _translate("ImportWindow", "whiteGrayTabWidget"))
        self.minSlider_Label.setText(_translate("ImportWindow", "Min :"))
        self.minSlider_Label.setProperty("class", _translate("ImportWindow", "standardLabel"))
        self.maxSlider_Label.setText(_translate("ImportWindow", "Max :"))
        self.maxSlider_Label.setProperty("class", _translate("ImportWindow", "standardLabel"))
        self.threshold_Label.setText(_translate("ImportWindow", "Threshold :"))
        self.threshold_Label.setProperty("class", _translate("ImportWindow", "infoLabel"))
        self.rank_Label.setText(_translate("ImportWindow", "Rank :"))
        self.rank_Label.setProperty("class", _translate("ImportWindow", "infoLabel"))
        self.rankValue_Label.setProperty("class", _translate("ImportWindow", "valueLabel"))
        self.limitedGraph_TabWidget.setTabText(self.limitedGraph_TabWidget.indexOf(self.threshodTab_Widget), _translate("ImportWindow", "Threshold"))
        self.percentageTable_TableWidget.setProperty("class", _translate("ImportWindow", "infoTable"))
        item = self.percentageTable_TableWidget.horizontalHeaderItem(0)
        item.setText(_translate("ImportWindow", "Percentage"))
        item = self.percentageTable_TableWidget.horizontalHeaderItem(1)
        item.setText(_translate("ImportWindow", "ABS Mean"))
        self.nbTable_TableWidget.setProperty("class", _translate("ImportWindow", "infoTable"))
        item = self.nbTable_TableWidget.horizontalHeaderItem(0)
        item.setText(_translate("ImportWindow", "Rank"))
        item = self.nbTable_TableWidget.horizontalHeaderItem(1)
        item.setText(_translate("ImportWindow", "ABS Mean"))
        self.limitedGraph_TabWidget.setTabText(self.limitedGraph_TabWidget.indexOf(self.tablesTab_Widget), _translate("ImportWindow", "Table"))
        self.importFiles_TabWidget.setProperty("class", _translate("ImportWindow", "whiteGrayTabWidget"))
        self.previousFiltersFiles_Label.setText(_translate("ImportWindow", "Previous Filters File Opened :"))
        self.previousFiltersFiles_Label.setProperty("class", _translate("ImportWindow", "standardLabel"))
        self.errorDataNameFile_Label.setProperty("class", _translate("ImportWindow", "errorLabel"))
        self.previousFlutFiles_List.setProperty("class", _translate("ImportWindow", "blackListWidget"))
        self.openedNameFile_Label.setText(_translate("ImportWindow", "No Name File Opened"))
        self.openedNameFile_Label.setProperty("class", _translate("ImportWindow", "standardLabel"))
        self.openedFlutFile_Label.setText(_translate("ImportWindow", "No Flut File Opened"))
        self.openedFlutFile_Label.setProperty("class", _translate("ImportWindow", "standardLabel"))
        self.errorFlutFile_Label.setProperty("class", _translate("ImportWindow", "errorLabel"))
        self.importFiltersFile_Button.setProperty("class", _translate("ImportWindow", "whiteBGButton"))
        self.importFlutFile_Button.setProperty("class", _translate("ImportWindow", "whiteBGButton"))
        self.openedFiltersFile_Label.setText(_translate("ImportWindow", "No Filters File Opened (Optional File)"))
        self.openedFiltersFile_Label.setProperty("class", _translate("ImportWindow", "standardLabel"))
        self.openedDataFile_Label.setText(_translate("ImportWindow", "No File Opened"))
        self.openedDataFile_Label.setProperty("class", _translate("ImportWindow", "standardLabel"))
        self.previousDataFiles_Label.setText(_translate("ImportWindow", "Previous Data File Opened :"))
        self.previousDataFiles_Label.setProperty("class", _translate("ImportWindow", "standardLabel"))
        self.importNameFile_Button.setProperty("class", _translate("ImportWindow", "whiteBGButton"))
        self.previousFlutFiles_Label.setText(_translate("ImportWindow", "Previous FLUT File Opened :"))
        self.previousFlutFiles_Label.setProperty("class", _translate("ImportWindow", "standardLabel"))
        self.importDataFile_Button.setProperty("class", _translate("ImportWindow", "whiteBGButton"))
        self.previousDataFiles_List.setProperty("class", _translate("ImportWindow", "blackListWidget"))
        self.errorFiltersFile_Label.setProperty("class", _translate("ImportWindow", "errorLabel"))
        self.previousFiltersFiles_List.setProperty("class", _translate("ImportWindow", "blackListWidget"))
        self.importFiles_TabWidget.setTabText(self.importFiles_TabWidget.indexOf(self.filesTab_Widget), _translate("ImportWindow", "Files Part"))
        self.errorValidation_Label.setProperty("class", _translate("ImportWindow", "errorLabel"))
        self.validation_Button.setText(_translate("ImportWindow", "Validation"))
        self.validation_Button.setProperty("class", _translate("ImportWindow", "grayRoundedButton"))
        self.openedCurrentFiles_Label.setText(_translate("ImportWindow", "File Opened\n"
"(Data / FLUT)"))
        self.openedCurrentFiles_Label.setProperty("class", _translate("ImportWindow", "standardLabel"))
        self.openedCurrentDataFile_Label.setText(_translate("ImportWindow", "Data File :"))
        self.openedCurrentDataFile_Label.setProperty("class", _translate("ImportWindow", "infoLabel"))
        self.currentDataFile_Label.setProperty("class", _translate("ImportWindow", "valueLabel"))
        self.openedCurrentFlutFile_Label.setText(_translate("ImportWindow", "FLUT File :"))
        self.openedCurrentFlutFile_Label.setProperty("class", _translate("ImportWindow", "infoLabel"))
        self.currentFlutFile_Label.setProperty("class", _translate("ImportWindow", "valueLabel"))
        self.nbConnections_Label.setText(_translate("ImportWindow", "Number of Connections :"))
        self.nbConnections_Label.setProperty("class", _translate("ImportWindow", "infoLabel"))
        self.nbNodes_Label.setText(_translate("ImportWindow", "Number of Nodes :"))
        self.nbNodes_Label.setProperty("class", _translate("ImportWindow", "infoLabel"))
        self.nbNodesValue_Label.setProperty("class", _translate("ImportWindow", "valueLabel"))
        self.sumAbsValue_Label.setProperty("class", _translate("ImportWindow", "valueLabel"))
        self.max_Label.setText(_translate("ImportWindow", "Max :"))
        self.max_Label.setProperty("class", _translate("ImportWindow", "infoLabel"))
        self.standardDeviation_Label.setText(_translate("ImportWindow", "Standard\n"
"Deviation"))
        self.standardDeviation_Label.setProperty("class", _translate("ImportWindow", "infoLabel"))
        self.sumAbs_Label.setText(_translate("ImportWindow", "Sum (ABS) :"))
        self.sumAbs_Label.setProperty("class", _translate("ImportWindow", "infoLabel"))
        self.standardDeviationValue_Label.setProperty("class", _translate("ImportWindow", "valueLabel"))
        self.maxValue_Label.setProperty("class", _translate("ImportWindow", "valueLabel"))
        self.nbConnectionsValue_Label.setProperty("class", _translate("ImportWindow", "valueLabel"))
        self.meanAbs_Label.setText(_translate("ImportWindow", "Mean (ABS) :"))
        self.meanAbs_Label.setProperty("class", _translate("ImportWindow", "infoLabel"))
        self.meanAbsValue_Label.setProperty("class", _translate("ImportWindow", "valueLabel"))
        self.minValue_Label.setProperty("class", _translate("ImportWindow", "valueLabel"))
        self.min_Label.setText(_translate("ImportWindow", "Min :"))
        self.min_Label.setProperty("class", _translate("ImportWindow", "infoLabel"))



import os
from pathlib import Path

from PyQt5 import QtWidgets
from PyQt5.QtCore import Qt
from PyQt5.QtGui import QPixmap, QIcon, QGuiApplication
from PyQt5.QtWidgets import QDesktopWidget, QMessageBox, QPushButton

from matplotlib.backends.backend_qt5agg import FigureCanvasQTAgg as FigureCanvas
from matplotlib.figure import Figure

from src.View.ui.ImportWindow_ui import Ui_ImportWindow
from src.Controller.ImportFile_Controller import ImportFile_Controller


class ImportFile_View(QtWidgets.QMainWindow, Ui_ImportWindow):
    """
    A class to initialize the view of ImportFile Window
    """

    def __init__(self, parent=None):
        """
        Initializes all view elements and like actions to buttons
        """
        
        super(ImportFile_View, self).__init__(parent=parent)

        # Load UI Interface
        self.setupUi(self)

        # Load Controler File
        self.importFile_Controller = ImportFile_Controller(self)

        # Action Configuration
        self.InitAction()

        # Hidden Elements
        self.HiddenElements()

        # Images Loading
        self.InitLoadingImages()

        # Graph Preparation
        self.InitGraphPreparation()

    def InitAction(self):
        """
        Associate actions to widgets when the application is opened.

        This method configures various actions such as resizing the window,
        enabling mouse tracking on file list widgets, and assigning actions 
        to tool buttons.
        """

        # Resize the Window (90%)
        self.ResizeWindow()

        # Display Full Path when the mouse entered the item
        self.previousDataFiles_List.setMouseTracking(True)
        self.previousFlutFiles_List.setMouseTracking(True)
        self.previousFiltersFiles_List.setMouseTracking(True)

        # Asssociate Action to ToolButton
        self.importDataFile_Button.setDefaultAction(self.importFile_Controller.OpenDataFile_Qaction)
        self.importNameFile_Button.setDefaultAction(self.importFile_Controller.OpenNameFile_Qaction)
        self.importFlutFile_Button.setDefaultAction(self.importFile_Controller.OpenFlutFile_Qaction)
        self.importFiltersFile_Button.setDefaultAction(self.importFile_Controller.OpenFiltersFile_Qaction)
        self.validation_Button.setDefaultAction(self.importFile_Controller.Validation_Qaction)

    def HiddenElements(self):
        """
        Hide specific elements when the application is opened.

        This method hides labels, buttons, and the graph section that should
        not be visible until certain actions are performed by the user.
        """

        # Label / Button to import Name File
        self.openedNameFile_Label.hide()
        self.importNameFile_Button.hide()

        self.errorDataNameFile_Label.hide()
        self.errorFlutFile_Label.hide()
        self.errorFiltersFile_Label.hide()

        # Graph Section
        self.graphSection_Widget.hide()

    def InitLoadingImages(self):
        """
        Load images and associate them with the tool buttons.

        This method loads an image from the resources directory, scales it, 
        and sets it as an icon for multiple tool buttons.
        """

        resourcedir = Path(__file__).parent.parent.parent / 'resources'
        imagePath = os.path.join(resourcedir, "images", "folder.png")

        # Load image using the variable path
        pixmap = QPixmap(imagePath)
        pixmap = pixmap.scaled(self.importDataFile_Button.width(), self.importDataFile_Button.height(),
                               Qt.KeepAspectRatioByExpanding)

        # Check if pixmap loaded successfully
        if not pixmap.isNull():

            # Set pixmap as icon for the tool button
            icon_Qicon = QIcon(pixmap)

            self.importDataFile_Button.setIcon(icon_Qicon)
            self.importDataFile_Button.setIconSize(pixmap.size())
            self.importNameFile_Button.setIcon(icon_Qicon)
            self.importNameFile_Button.setIconSize(pixmap.size())
            self.importFlutFile_Button.setIcon(icon_Qicon)
            self.importFlutFile_Button.setIconSize(pixmap.size())
            self.importFiltersFile_Button.setIcon(icon_Qicon)
            self.importFiltersFile_Button.setIconSize(pixmap.size())

        else:
            print("[ERROR] Image Not Correctly Loaded")
    
    def InitGraphPreparation(self):
        """
        Create a Matplotlib figure and a canvas to display histograms.

        This method initializes a Matplotlib figure and embeds it within the 
        GUI for displaying histograms or other graphical data.
        """

        # Create a Matplotlib figure and a canvas to display it
        self.graph = Figure()
        self.canvas = FigureCanvas(self.graph)
        self.graphDisplay_Layout.addWidget(self.canvas)

    def ResizeWindow(self):
        """
        Resize the window to 90% of the screen size when the application is opened.

        This method calculates the new window dimensions based on the primary screen's 
        size and positions the window centrally.
        """

        # Obtain the Screen Size
        screen = QGuiApplication.primaryScreen().geometry()
        screenWidth = screen.width()
        screenHeight = screen.height()

        # Define the Window Size (80%)
        widthPercentage = 0.9
        heightPercentage = 0.9

        # Compute the new Window Size
        newWidth = int(screenWidth * widthPercentage)
        newHeight = int(screenHeight * heightPercentage)

        startX = int((screenWidth * (1 - widthPercentage)) / 2)
        startY = int((screenHeight * (1 - widthPercentage)) / 2)

        # Apply new Value
        #self.resize(newWidth, newHeight)
        self.setGeometry(startX, startY, newWidth, newHeight)

    def LoadIcon(self, iconType):
        """
        Load an icon based on the specified type.

        Parameters
        ----------
        iconType : str
            The type of icon to load (warning).

        Returns
        -------
        QIcon
            The loaded QIcon object.
        """

        resourcedir = Path(__file__).parent.parent.parent / 'resources'
        imagePath = os.path.join(resourcedir, "images", iconType + ".png")

        return QIcon(imagePath)


    # ===========================
    # ===== Warnings Popups =====
    # ===========================

    def WarningPopUpFilters(self, filtersToIgnore):
        """
        Display a warning popup about filter incoherences.

        Parameters
        ----------
        filtersToIgnore : list of bool
            List indicating which filters are incoherent.
        """

        msgBox = QMessageBox(self)
        msgBox.setWindowTitle("Coherence Filters")

        # Display a different text depending of the incoherent (and theirs numbers)
        if filtersToIgnore.count(True) > 1:
            text = "Data and FLUT files are not coherent. \
                        \nDo you want ignore Incoherences ?"
        else:
            match filtersToIgnore.index(False)[0]:
                case 0:
                    text = "Data and FLUT files are not coherent. \
                        \nDo you want ignore Pre-Filter Threshold Incoherence ?"
                case 1:
                    text = "Data and FLUT files are not coherent. \
                        \nDo you want ignore Relative Threshold Incoherence ?"
                case 2:
                    text = "Data and FLUT files are not coherent. \
                        \nDo you want ignore Relative Threshold Incoherence ?"
                case 3:
                    text = "Data and FLUT files are not coherent. \
                        \nDo you want ignore Absolu Threshold Incoherence ?"
                case 4:
                    text = "Data and FLUT files are not coherent. \
                        \nDo you want ignore Absolu Threshold Incoherence ?"
                case 5:
                    text = "Data and FLUT files are not coherent. \
                        \nDo you want ignore Rank Threshold Incoherence ?"
                case 6:
                    text = "Data and FLUT files are not coherent. \
                        \nDo you want ignore Inter-Region Incoherence(s) ?"
                    
        msgBox.setText(text)
        msgBox.setWindowIcon(self.LoadIcon("warning"))
        
        ignoreButton = QPushButton("Ignore")
        cancelButton = QPushButton("Cancel")

        msgBox.addButton(ignoreButton, QMessageBox.ActionRole)
        msgBox.addButton(cancelButton, QMessageBox.ActionRole)

        # Connect button to ImportFile Controller Methods
        ignoreButton.clicked.connect(self.importFile_Controller.IgnoreFiltersIncoherences)
        cancelButton.clicked.connect(self.importFile_Controller.CancelFilters)

        msgBox.exec_()

    def WarningPopUpNumberRegion(self):
        """
        Displays a warning popup about the number of regions.

        Alerts the user that the number of regions is higher than the recommended 
        limit. The user can choose to display all regions or hide those not present 
        in the data file.
        """

        msgBox = QMessageBox(self)
        msgBox.setWindowTitle("Region Number")
        msgBox.setText("The number of regions is higher than the recommended number (150 with blanks). \
                       \nDo you want to DISPLAY all regions, or HIDE regions not present in data file ?")
        msgBox.setWindowIcon(self.LoadIcon("warning"))
        
        displayButton = QPushButton("Display")
        hideButton = QPushButton("Hide")

        msgBox.addButton(displayButton, QMessageBox.ActionRole)
        msgBox.addButton(hideButton, QMessageBox.ActionRole)

        # Connect button to ImportFile Controller Methods
        displayButton.clicked.connect(self.importFile_Controller.DisplayAllRegions)
        hideButton.clicked.connect(self.importFile_Controller.HideRegions)

        msgBox.exec_() 
    
    def WarningPopUpDATAFLUT(self):
        """
        Display a warning popup about incoherences between Data and FLUT files.

        This method prompts the user to either ignore the incoherences or cancel 
        the validation process.
        """

        msgBox = QMessageBox(self)
        msgBox.setWindowTitle("Coherence Verification")
        msgBox.setText("Data and FLUT files are not coherent. \
                       \nDo you want ignore Incoherence(s) ?")
        msgBox.setWindowIcon(self.LoadIcon("warning"))
        
        ignoreButton = QPushButton("Ignore")
        cancelButton = QPushButton("Cancel")

        msgBox.addButton(ignoreButton, QMessageBox.ActionRole)
        msgBox.addButton(cancelButton, QMessageBox.ActionRole)

        # Connect button to ImportFile Controller Methods
        ignoreButton.clicked.connect(self.importFile_Controller.IgnoreDATAFLUTIncoherences)
        cancelButton.clicked.connect(self.importFile_Controller.CancelFLUT)

        msgBox.exec_()


    # =======================
    # ===== Close Event =====
    # =======================
    
    def closeEvent(self, event):
        """
        Handle the event when the window is closed.

        This method ensures that when the application is closed, 
        the controller is called to save necessary data.

        Parameters
        ----------
        event : QCloseEvent
            The close event triggered when the window is closed.
        """

        # when the application is closed, called the control to save some data
        if hasattr(self, 'importFile_Controller'):
            self.importFile_Controller.CloseImportWithoutValidation()



from PyQt5.QtWidgets import QTableWidgetItem

class NumericTableWidgetItem(QTableWidgetItem):
    """
    A QTableWidgetItem subclass for handling numeric values in a QTableWidget.
    """

    def __init__(self, value):
        """
        Initialize the NumericTableWidgetItem with a numeric value.

        Converts the numeric value to a string for display purposes 
        and stores the original value for numeric comparisons.

        Parameters
        ----------
        value : int, float
            The numeric value to initialize the table item with.
        """

        super().__init__(str(value))
        self.value = value

    def __lt__(self, other):
        """
        Less-than comparison operator for numeric sorting.

        Compares the numeric value of this item with another 
        NumericTableWidgetItem to determine their order.

        Parameters
        ----------
        other : NumericTableWidgetItem
            The other table item to compare against.

        Returns
        -------
        bool
            True if this item's value is less than the other item's value, False otherwise.
        """

        if isinstance(other, NumericTableWidgetItem):
            return self.value < other.value
        return super().__lt__(other)


import os
from pathlib import Path

class ExportPrevious_Files:
    """
    A class to Export Previous Files stored in Project Object.
    Previous files represent all files opened during the application sessions (limited to 5).
    """

    def __init__(self):
        resourcedir = Path(__file__).parent.parent.parent / 'resources'
        self.backupFiles = os.path.join(resourcedir, "Backup_PreviousFiles.txt")

    def SaveFile(self, previousFiles: dict):
        """
        Save information about previous files into a backup file.

        Parameters
        ----------
        previousFiles : dict
            A dictionary containing previous files categorized by type.
            Expected structure:
            - "DataFiles": dict of {filename: filetype}
            - "NameFiles": dict of {DataFilename: NameFile}
            - "FlutFiles": list of filenames
            - "FiltersFiles": list of filenames
            - "ProjectFiles": list of filenames

        Raises
        ------
        ValueError
            If the previousFiles dictionary is not in the expected format.
        FileNotFoundError
            If the backup file cannot be created or opened.
        """

        try:
            backupFilenames = open(os.path.abspath(self.backupFiles), 'w')
        except FileNotFoundError:
            raise Exception("Creation Error - {}".format(self.backupFiles))
        
        # Write all Previous File in the Backup File

        # DATA File Part
        backupFilenames.write("DataFiles\n")
        
        for dataFile, fileType in previousFiles["DataFiles"].items():
            backupFilenames.write(dataFile + "|" + fileType + "\n")

        # Name File Part
        backupFilenames.write("NameFiles\n")
        
        for dataFile, nameFile in previousFiles["NameFiles"].items():
            backupFilenames.write(dataFile + "|" + nameFile + "\n")

        # Flut File Part
        backupFilenames.write("FlutFiles\n")

        for flutFile in previousFiles["FlutFiles"]:
            backupFilenames.write(flutFile + "\n")

        # Filters File Part
        backupFilenames.write("FiltersFiles\n")
        
        for filtersFile in previousFiles["FiltersFiles"]:
            backupFilenames.write(filtersFile + "\n")

        # PROJECT File Part
        backupFilenames.write("ProjectFiles\n")
        
        for projectFile in previousFiles["ProjectFiles"]:
            backupFilenames.write(projectFile + "\n")

        backupFilenames.close()



from PyQt5.QtCore import Qt, pyqtSignal
from PyQt5.QtWidgets import QLabel, QWidget, QGraphicsView, QGraphicsScene, QGraphicsProxyWidget, QVBoxLayout


class CheckGridLabel_Widget(QWidget):
    """
    A custom QWidget that displays a label with a specified name and color, and allows for rotation.
    """

    name : str
    clicked = pyqtSignal(str)
    orientation : str
    checked : bool = True

    def __init__(self, name, orientation, color, rotation=0):
        """
        Initializes the CheckGridLabel_Widget instance.

        Parameters
        ----------
        name : str
            The name to be displayed on the label.
        orientation : str
            The orientation of the label.
        color : tuple of float
            The RGB color values of the label in the range [0, 1].
        rotation : int, optional
            The rotation angle of the label in degrees. Default is 0.

        Raises
        ------
        ValueError
            If the color parameter does not have exactly three elements.
        """

        super().__init__()

        self.name = name
        self.orientation = orientation

        self.setContentsMargins(0, 0, 0, 0)

        self.view = QGraphicsView(self)
        self.view.setHorizontalScrollBarPolicy(Qt.ScrollBarAlwaysOff)
        self.view.setVerticalScrollBarPolicy(Qt.ScrollBarAlwaysOff)
        self.view.setStyleSheet("background: transparent;"
                                "border : 0px;")  # Set QGraphicsView background to transparent
        self.view.setAlignment(Qt.AlignTop | Qt.AlignLeft)
        self.scene = QGraphicsScene(self.view)
        self.view.setScene(self.scene)

        # Add two spaces around the name
        label = QLabel(f" {name} ")
        label.setProperty("class", "standardLabel")
        label.setStyleSheet("background: transparent;")  # Set QLabel background to transparent

        # Convert 0 to 1 color values into 0 to 255 values
        color = [int(color * 255) for color in color]
        color = f"{color[0]}, {color[1]}, {color[2]}"
        label.setStyleSheet(f"background-color : rgb({color});")
        label.adjustSize()

        proxy = QGraphicsProxyWidget()
        proxy.setWidget(label)

        # Apply rotation transformation
        proxy.setTransformOriginPoint(label.rect().center())
        proxy.setRotation(rotation)

        self.scene.addItem(proxy)

        layout = QVBoxLayout(self)
        layout.setContentsMargins(0, 0, 0, 0)
        layout.addWidget(self.view)

        match rotation :
            case 0 | 180:
                self.setFixedHeight(label.height())
            case 45 | 135 | 225 | 315:
                self.setFixedWidth(label.width() ** (1/2))
                self.setFixedHeight(label.width() ** (1/2))
            case 90 | 270:
                self.setFixedWidth(label.height())

    def mousePressEvent(self, ev):
        """
        Handles mouse press events by emitting the clicked signal with the object's name.

        Parameters
        ----------
        ev : QMouseEvent
            The mouse event containing information about the mouse action.
        """
        
        # Emit object name
        self.clicked.emit(self.objectName())



import os
from pathlib import Path

from PyQt5.QtCore import Qt
from PyQt5.QtGui import QIcon
from PyQt5.QtWidgets import QDialog, QLabel, QVBoxLayout

class LoadingPopup(QDialog):
    """
        A class handle Loading Popup. Informs the user of Load
    """

    def __init__(self, parent=None):
        """
        Creates popup with style sheet and disable close and help button.
        """
        super().__init__(parent)

        # Remove Help Button and Disable Close Windows
        self.setWindowFlags(Qt.Window | Qt.WindowTitleHint | Qt.WindowSystemMenuHint)

        # Add Style to Qdialog
        resourcedir = Path(__file__).parent.parent.parent / 'resources'
        with open(os.path.join(resourcedir, "Style_Application.qss"), 'r') as file:
            stylesheet = file.read()
            self.setStyleSheet(stylesheet)

        # Set Title and Window Size
        self.setWindowTitle("Loading")
        self.setModal(True)
        self.setFixedSize(200, 100)

        # Add Label for the text
        layout = QVBoxLayout()
        self.label = QLabel("Loading...", self)
        self.label.setProperty("class", "loadingPopup")

        layout.addWidget(self.label)
        self.setLayout(layout)
        